from mpl_playback.playback import playback_file

# playback_file("_dynamic-playback.json", "dynamic.py", "ohboi.gif")
# playback_file("_subplots-playback.json", "subplots.py", "ohboi.gif")
playback_file("_multifig-playback.json", "multifig.py", ["multi1.gif", "multi2.gif"])
# playback_file("_multifig-playback.json", "multifig.py", ["multi1-garbo.gif", "multi2-garbo.gif"])

# playback_file("_multifig-playback.json", "multifig.py", ["multi.gif"])
